package com.biganto.visual.roompark.base

import android.os.Bundle
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.widget.Toolbar
import androidx.coordinatorlayout.widget.CoordinatorLayout
import butterknife.BindView
import butterknife.ButterKnife
import com.biganto.visual.roompark.BuildConfig
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.presentation.screen.splash.SplashScreenController
import com.biganto.visual.roompark.util.view_utils.app_bar.DragControlAppBarLayoutBehaviour
import com.bluelinelabs.conductor.Conductor
import com.bluelinelabs.conductor.Router
import com.bluelinelabs.conductor.RouterTransaction
import com.crashlytics.android.Crashlytics
import com.google.android.material.appbar.AppBarLayout
import com.google.android.material.appbar.CollapsingToolbarLayout
import com.google.android.material.bottomnavigation.BottomNavigationView
import io.fabric.sdk.android.Fabric
import kotlinx.android.synthetic.main.activity_main.*
import timber.log.Timber

class RoomParkMainActivity(
) : BaseRoomParkActivity()
    ,ICollapsingToolBar
    ,IConductorActivity
    ,IBottomNavigation{



//    @Inject
//    lateinit var snackbarProvider: ISnackBarProvider

    private lateinit var router: Router

    @BindView(R.id.top_toolbar) override lateinit var topAppBar: Toolbar
    @BindView(R.id.app_bar) override lateinit var appBar: AppBarLayout
    @BindView(R.id.topToolbarHolder) override lateinit var coordinatorLayout: CoordinatorLayout
    @BindView(R.id.collapsingToolbarLayout) override lateinit var collapsingToolbarLayout: CollapsingToolbarLayout
    @BindView(R.id.conductor_container) override lateinit var conductorContainer: ViewGroup
    @BindView(R.id.bottom_navigation_view) override lateinit var bottomNavigation: BottomNavigationView

    @BindView(R.id.bottom_view_divider) lateinit var bottomShadow: View

    override fun onCreate(savedInstanceState: Bundle?) {
        setTheme(R.style.AppTheme)
        super.onCreate(savedInstanceState)

        if (!BuildConfig.DEBUG) Fabric.with(this, Crashlytics())

        setContentView(R.layout.activity_main)

        ButterKnife.bind(this)

        setSupportActionBar(topAppBar)

        hideAll()

        router = Conductor.attachRouter(this, conductorContainer, savedInstanceState)

        router.setRoot(RouterTransaction.with(SplashScreenController()))
    }

    override fun displayBackButton(show: Boolean) {
        supportActionBar?.setDisplayHomeAsUpEnabled(show)
    }

    override fun hideAll() {
        appBar.visibility=Toolbar.INVISIBLE
        Timber.e(" lay params : ${appBar.layoutParams.javaClass}")
        val params:CoordinatorLayout.LayoutParams = conductor_container.layoutParams as CoordinatorLayout.LayoutParams
        params.behavior = null
        conductor_container.requestLayout()
    }

    override fun hide() {
        bottomNavigation.visibility = BottomNavigationView.GONE
        bottomShadow.visibility = View.GONE
    }

    override fun show() {
        bottomNavigation.visibility = Toolbar.VISIBLE
        bottomShadow.visibility = View.VISIBLE
    }



    override fun showAll() {
        appBar.setExpanded(true,false)
        collapsingToolbarLayout.visibility= View.VISIBLE
        appBar.visibility= View.VISIBLE
        topAppBar.visibility = View.VISIBLE

        val params = conductor_container.layoutParams as CoordinatorLayout.LayoutParams
        params.behavior = AppBarLayout.ScrollingViewBehavior()
        conductor_container.requestLayout()
    }

    override fun appBarScrollable(allow: Boolean) {
        val params = appBar.layoutParams as CoordinatorLayout.LayoutParams
        val behavior = params.behavior as DragControlAppBarLayoutBehaviour
        behavior.allowDrag=allow
    }

    override fun onBackPressed() {
        if (router.handleBack()) {
            Timber.d("In stack : " + router.backstackSize)
        } else {
            super.onBackPressed()
        }
    }
}
