package com.biganto.visual.roompark.data.local

import android.app.Application
import com.afollestad.rxkprefs.Pref
import com.afollestad.rxkprefs.rxkPrefs
import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import dagger.Module
import dagger.Provides
import io.reactivex.Completable
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Singleton

/**
 * Created by Vladislav Bogdashkin on 14.06.2018.
 */

@Module
class LocalStorage(){

//    @Binds
//    @Singleton
//    abstract fun provideStore(store:UserHolder):ILocalStore


    @Provides
    @Singleton
    fun provideStore(app : Application):ILocalStore = UserHolder(app)
}

class UserHolder @Inject constructor(val context : Application) : ILocalStore
{
    init {
        Timber.d(" inited")
    }
    private val prefs = rxkPrefs(context)

    private val userSession : Pref<String> = prefs.string(RECENT_UUID_KEY, EMPTY_PREF_VALUE_KEY)

    companion object {
        const val RECENT_UUID_KEY = "com.biganto.visual.androidplayer.LAST_USER_UUD"
        const val EMPTY_PREF_VALUE_KEY = "NO_ACTIVE_SESSION"

        const val SCROLL_LIST_TO_TOUR_KEY="SCROLL_LIST_TO_TOUR_ID"


    }

    override fun setDownloadListDestinationTourId(tourId:String) =
            prefs.string(SCROLL_LIST_TO_TOUR_KEY,tourId)
                .observe()

    override fun getDownloadListDestinationTourId() =
            prefs.string(SCROLL_LIST_TO_TOUR_KEY,"")
                .observe()

    override fun recentUserObs() = userSession.observe()

    override fun recentUser(): Observable<in UserState> =
        userSession
                .observe()
                    .map {
                        Timber.d(" AUTH VALUE: $it")
                        Timber.d(" EMPTY_PREF_VALUE_KEY: $EMPTY_PREF_VALUE_KEY")
                        if (it == EMPTY_PREF_VALUE_KEY) return@map UserState.NotAuthenticated()
                        else return@map UserState.Authenticated(it)
                    }

    override fun setRecentUser(uuid: String?) =
        Completable.defer {
            Completable.fromCallable { userSession.set(uuid ?: EMPTY_PREF_VALUE_KEY) }
                .doOnComplete { Timber.d("complete save null value") }
        }
}

sealed class UserState{
    class NotAuthenticated
    data class Authenticated(val uuid: String)
}

data class UserPrefModel(
        val uuid:String
        ,val name:String
        ,val email:String
        ,val timeZone:String)
