package com.biganto.visual.roompark.data.repository.db

import com.biganto.visual.roompark.data.repository.db.requrey.model.*
import io.reactivex.Observable
import io.reactivex.Single
import io.requery.Persistable
import io.requery.reactivex.ReactiveResult

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */
interface IDb {
    fun upsertUser(entity: UserEntity): Observable<UserEntity>?
    fun <T : Persistable> upsert(entity: T): Single<T>
    fun <T : List<Persistable>> upsert(entity: T): Single<Iterable<Persistable>>
    fun fetchFeeds(): Observable<FeedEntity>
    fun fetchUser(uuid: Int): Observable<UserEntity>
    fun <T : List<Persistable>> blockingUpsert(entity: T)
    fun userObservableResult(uuid: Int): Observable<ReactiveResult<UserEntity>>
    fun fetchArticles(feedAlias: String, pageSize: Int, startIndex: Int): Observable<ArticleEntity>
    fun getTopLevelAlbums(): Observable<List<ImageAlbumEntity>>
    fun getChildAlbums(parentId: Int): Observable<List<ImageAlbumEntity>>
    fun checkIfExistsAlbumJunction(albumId: Int, parentAlbumId: Int): ImageAlbumJunctionEntity?
    fun getFeed(feedAlias: String): ReactiveResult<FeedEntity>
    fun getArticle(id: Int): ReactiveResult<ArticleEntity>
    fun getPhotos(albumId: Int): Observable<GalleryPhotoEntity>
    fun getPhoto(photoId: Int): Observable<GalleryPhotoEntity>
    fun getAlbum(albumId: Int): Observable<ImageAlbumEntity>
    fun getUserFavorites(uuid: Int): Observable<EstateEntity>
    fun fetchAllUsers(): Observable<List<UserEntity>>
    fun getEstate(estateId: Int): Observable<EstateEntity>
}