package com.biganto.visual.roompark.data.repository.file

import android.app.Application
import android.content.Context
import android.os.Environment
import com.biganto.visual.roompark.util.extensions.asInt
import com.google.gson.JsonElement
import dagger.Module
import timber.log.Timber
import java.io.File
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 29.06.2018.
 */


@Module
class FileModule @Inject constructor(val context: Application) {

    private val rootFolder: File
        get() {
            val rootFile = context.filesDir
            Environment.getDataDirectory()
            if (!rootFile.exists())
                rootFile.mkdirs()
            return rootFile
        }


    fun getFile(fileUri: String): File {
        try {
            val fileName =if (fileUri.contains("/")) fileUri.substring(fileUri.lastIndexOf("/")) else fileUri
            val fileDir = if (fileUri.contains("/")) fileUri.substring(0, fileUri.lastIndexOf("/")) else ""
            val directory = File(rootFolder, fileDir)
            directory.mkdirs()

            val file = File(directory, fileName)
            return file

        }
        catch (e:Exception)
        {
            Timber.e("Caused file: %s",fileUri)
            throw error("unconditional file")
        }
    }

    fun saveFileToDisk(file:File,jsonElement: JsonElement){
        file.writeText("[$jsonElement]") //to json array because core unity method parse data like TourData[] Estate[] etc..
    }

    fun saveFileToDisk(file:File,content: String){
        Timber.d("write to : $file")
        Timber.d("write to : ${file.name}")
//        file.createNewFile()
        file.parentFile.mkdirs()
        file.writeText(content) //to json array because core unity method parse data like TourData[] Estate[] etc..
    }

    fun deleteFile(uri:String)= getFile(uri).delete()

    val getCoreCacheDirectory:String
    get(){
        return rootFolder.absolutePath
    }

    companion object {
        fun getDirectory(context: Context, dirType: FileDirectory): File =
            File(context.filesDir.absolutePath.plus(dirType.dir))

        fun assetsDirectory(context: Context): String = context.filesDir.absolutePath
    }

    sealed class FileDirectory(val dir:String) {
        class PlanTypeDir: FileDirectory{
            constructor (dir:String = "/estates",childName:String? = null) : super(dir.plus("/$childName"))

            constructor (catalog:String = "/estates"
                            ,estateId: Int
                         , planId:Int
                         , furniture:Boolean? = null
                         , sizes:Boolean? = null
                         , walls:Boolean? = null
                         , electric:Boolean? = null
            ) : super(catalog.plus("/plan").plus("/$estateId").plus("/$planId")
                .plus("/plan${planId}" +
                        "${sizes?.asInt?.toString()?.map { "_s$it" }?:""}" +
                        "${furniture?.asInt?.toString()?.map { "_f$it" }?:""}" +
                        "${walls?.asInt?.toString()?.map { "_w$it" }?:""}" +
                        "${electric?.asInt?.toString()?.map { "_e$it" }?:""}")
                .plus(".html"))
        }

        class ToursDir(dir:String = "/tours",childName:String? = null) : FileDirectory(dir.plus("/childName"))
        class FeedsDir(dir:String = "/feeds",childName:String? = null) : FileDirectory(dir.plus("/childName"))
        class Albums(dir:String = "/photos",childName:String? = null) : FileDirectory(dir.plus("/childName"))
    }
}