package com.biganto.visual.roompark.data.repository.mapper

import android.content.res.Resources
import com.biganto.visual.roompark.data.repository.api.retrofit.raw.*
import com.biganto.visual.roompark.data.repository.db.requrey.PhotoResolutions
import com.biganto.visual.roompark.data.repository.db.requrey.TitledPhoto
import com.biganto.visual.roompark.data.repository.db.requrey.model.*
import kotlin.math.max

/**
 * Created by Vladislav Bogdashkin on 06.11.2019.
 */

private val display = Resources.getSystem().displayMetrics

fun fromRaw(raw: AuthRaw) : UserEntity {
    val user = UserEntity()
    user.setEmail(raw.email)
    user.setUuid(raw.id)
    user.setAuthToken(raw.token)
    user.setName(raw.name)
    user.setTargetResolution(calcTargetResolution)
    return user
}

fun fromRaw(raw:FeedRaw) : FeedEntity {
    val entity  = FeedEntity()
    entity.setTitle(raw.title)
    entity.setAlias(raw.alias)
    return entity
}

fun fromRaw(raw:ArticleRaw) : ArticleEntity {
    val entity  = ArticleEntity()
    entity.setId(raw.id)
    entity.setPublished(raw.published)
    entity.setTitle(raw.title)
    entity.setBody(raw.body)
    entity.setAnnounce(raw.announce)
    entity.setPreview(raw.preview)
    raw.photo?.let {entity.photo = fromRawList(it, ::fromRaw)}
    val feed = FeedEntity()
    feed.setAlias(raw.feed_alias)
    entity.setFeed(feed)

    return entity
}


fun fromRaw(raw:NewsArticleRaw,feedAlias:String):ArticleEntity{
    val entity = ArticleEntity()
    entity.setId(raw.id)
    entity.setPreview(raw.preview)
    entity.setAnnounce(raw.announce)
    entity.setPublished(raw.published)
    entity.setTitle(raw.title)
    val feed = FeedEntity()
    feed.setAlias(feedAlias)
    entity.setFeed(feed)
    return entity
}


fun fromRaw(raw:NewsPhotoRaw):TitledPhoto{
    return TitledPhoto(raw.title,raw.url)
}


fun fromRaw(raw:ImageAlbumRaw) : ImageAlbumEntity {
    val entity  = ImageAlbumEntity()
    entity.setId(raw.id)
    entity.setTitle(raw.title)
    entity.setPublished(raw.date)
    entity.setSort(raw.sort)
    entity.setPreview(raw.preview?:"https://room-park.ru/assets/gallery_images/image_2600/00/00/03/800-1b7546.png")
    return entity
}


fun fromRaw(raw:GalleryImageRaw) : GalleryPhotoEntity {
    val entity  = GalleryPhotoEntity()
    entity.setId(raw.id)
    entity.setTitle(raw.title)
    entity.setDescription(raw.description)
    entity.setSort(raw.sort)
    entity.setAlbum_id(raw.album_id)
    entity.resolutions = fromRawList(raw.resolutions,::fromRaw)
    return entity
}

fun fromRaw(raw:ResolutionRaw) =
    PhotoResolutions(raw.res_name,raw.url,raw.width,raw.height)




fun fromRaw(raw:EstateRaw):EstateEntity{
    val entity = EstateEntity()
    entity.setId(raw.id)
    entity.setType(raw.type)
    entity.setNumber(raw.number)
    entity.setSectionBegin(raw.common_info.section_begin)
    entity.setSectionEnd(raw.common_info.section_end)
    entity.setPlanJpgUrl(raw.plan_jpg?.url)
    entity.setPlanJpgWidth(raw.plan_jpg?.width)
    entity.setPlanJpgHeight(raw.plan_jpg?.height)
    entity.setPlanPngUrl(raw.plan_png?.url)
    entity.setPlanPngWidth(raw.plan_png?.width)
    entity.setPlanPngHeight(raw.plan_png?.height)
    entity.setRooms(raw.common_info.rooms)
    entity.setAlbumId(raw.album_id)
    entity.setMultitourId(raw.multitour_id)

    entity.setUrl(raw.url)
    entity.setInfo_floor_max(raw.common_info.floor_max)
    entity.setInfo_area(raw.common_info.area)
    entity.setInfo_area_living(raw.common_info.area_living)
    entity.setInfo_kind(raw.common_info.kind)
    entity.setInfo_dependent(raw.common_info.dependent)
    entity.setInfo_decoration(raw.common_info.decoration)
    entity.setInfo_building(raw.common_info.building)
    entity.setInfo_section_begin(raw.common_info.section_begin)
    entity.setInfo_floor(raw.common_info.floor)

    entity.setInfo_window_face(raw.common_info.windows_face)
    entity.setInfo_price(raw.common_info.price)
    entity.setInfo_price_meter(raw.common_info.price_meter)
    entity.setInfo_ceiling(raw.common_info.ceiling)
    entity.setInfo_direction(raw.common_info.direction)
    entity.setFavorite(false)
    return entity
}

fun fromRaw(raw:PlanTypeRaw):PlanPresetEntity{
    val entity = PlanPresetEntity()
    entity.setFeatures(raw.features)
    entity.setPlanId(raw.plan_id)
    entity.setTitle(raw.title)
    entity.explication.clear()
    entity.explication.addAll(fromRawList(raw.explication!!,::fromRaw))
    return entity
}

fun fromRaw(raw:EstateRoomRaw):ExplicationEntity{
    val  entity = ExplicationEntity()
    entity.setArea(raw.area)
    entity.setLiving(raw.living)
    entity.setTitle(raw.title)
    return entity
}


//fun fromRaw(raw: List<FeedRaw>):List<FeedEntity> = List(raw.displaySize) { index-> fromRaw(raw[index]) }


fun <E,M> fromRawList(raw: List<E>,block:(E)->M):List<M> = List(raw.size) { index-> block(raw[index]) }

fun fromRaw(raw: List<NewsArticleRaw>,feedAlias:String):List<ArticleEntity> =
    List(raw.size) { index-> fromRaw(raw[index],feedAlias) }

val calcTargetResolution = max(display.widthPixels,display.heightPixels)
