package com.biganto.visual.roompark.presentation.screen.auth

import android.view.View
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.IBottomNavigation
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.home.home_routing.HomeBottomNavigationController
import com.bluelinelabs.conductor.RouterTransaction
import com.google.android.material.button.MaterialButton
import com.google.android.material.textfield.TextInputLayout
import com.jakewharton.rxbinding3.view.clicks
import com.jakewharton.rxbinding3.widget.textChanges
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import timber.log.Timber
import java.util.concurrent.TimeUnit
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class AuthScreenController  :
    BigantoBaseController<AuthScreenViewState
        , AuthScreen
        , AuthScreenPresenter>()
    , AuthScreen {


    override fun onViewBound(v: View) {
        toolBar.hideAll()
        bottomNavigation.hide()
    }

    @BindView(R.id.login_text_input) lateinit var loginInput:TextInputLayout
    @BindView(R.id.password_text_input) lateinit var pwdInput:TextInputLayout
    @BindView(R.id.sign_in_button) lateinit var signInButton:MaterialButton

    override fun tryAuth(): Observable<AuthInputModel> =
            signInButton.clicks()
                .debounce (200L,TimeUnit.MILLISECONDS)
                .doOnNext { signInButton.hideKeyboard() }
                .map<AuthInputModel> {
                    AuthInputModel(
                        loginInput.editText?.text?.toString() ?: ""
                        , pwdInput.editText?.text?.toString() ?: ""
                    )
                }
                .observeOn(AndroidSchedulers.mainThread())

    override fun injectDependencies() {
        getComponent()
    }

    override fun onAttach(view: View) {
        super.onAttach(view)
        detachDisposable.addAll(
            loginInput.editText?.textChanges()
                ?.filter { loginInput.isErrorEnabled }
                ?.subscribe {
                    loginInput.isErrorEnabled = false
                    loginInput.error = null
                    signInButton.isEnabled=true
                },
            pwdInput.editText?.textChanges()
                ?.filter { pwdInput.isErrorEnabled }
                ?.subscribe {
                    pwdInput.isErrorEnabled = false
                    pwdInput.error = null
                    signInButton.isEnabled=true
                }
        )
    }

    @Inject
    lateinit var bottomNavigation: IBottomNavigation

    @Inject
    override lateinit var injectedPresenter: AuthScreenPresenter

//    @Inject
//    lateinit var snacky:ISnackBarProvider

    lateinit var rpActivity: RoomParkMainActivity

    fun getComponent() = DaggerAuthScreenComponent
        .factory()
        .create(
            RoomParkApplication.component
            ,activity as RoomParkMainActivity)
        .inject(this)

    override fun render(viewState: AuthScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is AuthScreenViewState.Idle -> render(viewState)
            is AuthScreenViewState.Authorization -> render(viewState)
            is AuthScreenViewState.SignedIn -> render(viewState)
            is AuthScreenViewState.SignInError -> render(viewState)
            is AuthScreenViewState.SomeError -> render(viewState)
            is AuthScreenViewState.WrongLogin -> render(viewState)
            is AuthScreenViewState.WrongPassword -> render(viewState)
        }
    }

    private fun render(viewState: AuthScreenViewState.Idle){
        loginInput.isErrorEnabled = false
        loginInput.error = null
        pwdInput.isErrorEnabled = false
        pwdInput.error = null
        signInButton.isEnabled=true
    }

    private fun render(viewState: AuthScreenViewState.Authorization){
//        toolBar.hideAll()
        signInButton.isEnabled = false
//        snacky.showSnackBar("lul")
    }

    private fun render(viewState: AuthScreenViewState.SignedIn){
        router.setRoot(RouterTransaction.with(HomeBottomNavigationController()))
//        snacky.showSnackBar("lul")
    }

    private fun render(viewState: AuthScreenViewState.SignInError){
        showError(viewState.exception)
        signInButton.isEnabled=true
    }

    private fun render(viewState: AuthScreenViewState.WrongLogin){
//        showError(viewState.exception)
        loginInput.isErrorEnabled = true
        loginInput.errorIconDrawable = null
        viewState.exception.selectHandler(
            {strId -> loginInput.error = resources?.getString(strId)},
            {message -> loginInput.error = message}
        )
    }

    private fun render(viewState: AuthScreenViewState.WrongPassword){
        pwdInput.isErrorEnabled = true
        pwdInput.errorIconDrawable = null
        viewState.exception.selectHandler(
            {strId -> pwdInput.error = resources?.getString(strId)},
            {message -> pwdInput.error = message}
        )
    }


    private fun render(viewState: AuthScreenViewState.SomeError) {
        showError(viewState.exception)
        signInButton.isEnabled=true
    }


    override fun getLayoutId(): Int = R.layout.authentication_screen


}