package com.biganto.visual.roompark.presentation.screen.deal

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.widget.LinearLayout
import androidx.core.os.bundleOf
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressAnimationState
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressCeil
import com.google.android.material.textview.MaterialTextView
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_DEAL_ID_KEY = "SELECTED_DEAL_GUID"


class DealScreenController  :
    BigantoBaseController<DealScreenViewState
        , DealScreen
        , DealScreenPresenter>
    , DealScreen {

    constructor(args: Bundle):super(args)

    constructor(id: String) : super(bundleOf(SELECTED_DEAL_ID_KEY to id))

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: DealScreenPresenter

    @BindView(R.id.deal_card_header)
    lateinit var dealTitle: MaterialTextView


    @BindView(R.id.statusContainer)
    lateinit var statusContainer: LinearLayout


    @BindView(R.id.progress_holder)
    lateinit var progressLayout: LinearLayout

    private fun setToolbar(){
        toolBar.showAll()
        toolBar.appBar.setExpanded(false,false)
        toolBar.collapsingToolbarLayout.title = "СДЕЛКА"
        toolBar.appBar.liftOnScrollTargetViewId = R.id.favorites_cards_recycler_view
        toolBar.appBar.setLiftable(true)
        toolBar.appBarScrollable(false)

//        planTypesTabLayout.
    }

    private fun bindRecycler() {
    }

    override fun onViewBound(v: View) {
        setToolbar()
        bindRecycler()
    }

    override fun render(viewState: DealScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is DealScreenViewState.Idle -> render(viewState)
            is DealScreenViewState.LoadDeal -> render(viewState)
            is DealScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: DealScreenViewState.Idle){

    }

    private fun render(viewState: DealScreenViewState.SomeError) =
        showError(viewState.exception)


    private fun render(viewState: DealScreenViewState.LoadDeal) {

        viewState.statusList.forEach {

            val statusLayout = LayoutInflater.from(activity)
                .inflate(R.layout.progress_status_title_vertical
                    ,progressLayout
                    ,false)
                    as LinearLayout

            Timber.d("layouted: $statusLayout")
            val statusCeil = statusLayout.findViewById<StatusProgressCeil>(R.id.status)
            Timber.d("layouted ceail : $statusCeil")

            val position = it.orderId
            val statusCount = viewState.statusList.size
            val lastStatusPassed = viewState.estate.statusId

            statusCeil.setHasEnd(position != statusCount)
            statusCeil.setHasStart(position > 1)
            statusCeil.setIsEnabled(position <= lastStatusPassed)
            statusCeil.setNext(position < lastStatusPassed)
            if (position > lastStatusPassed)
                statusCeil.setAnimState(StatusProgressAnimationState.DISABLE)
            statusCeil.invalidate()

            val statusTitle = statusLayout.findViewById<MaterialTextView>(R.id.title)
            Timber.d("layouted statusTitle : $statusTitle")
            statusTitle.text = it.title
            progressLayout.addView(statusLayout)
        }
        progressLayout.invalidate()


    }

        private fun getComponent() = DaggerDealScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity
            ,args.getString(SELECTED_DEAL_ID_KEY)?: error("Deal Id is empty!"))
        .inject(this)

    override fun getLayoutId(): Int = R.layout.deal_screen
}
