package com.biganto.visual.roompark.presentation.screen.deal

import android.content.Context
import androidx.annotation.StringRes
import com.biganto.visual.roompark.conductor.BigantoBasePresenter
import com.biganto.visual.roompark.domain.interactor.DealInteractor
import com.biganto.visual.roompark.domain.model.DealModel
import com.biganto.visual.roompark.domain.model.PlanPresetModel
import com.biganto.visual.roompark.presentation.screen.estate.InfoShowType
import com.biganto.visual.roompark.util.monades.ExceptionString
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject
import javax.inject.Named

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */


class DealScreenPresenter @Inject constructor(
    private val interactor: DealInteractor,
    private val context: Context,
    @Named(SELECTED_DEAL_ID_KEY) private val dealId:String
)
    : BigantoBasePresenter<DealScreen, DealScreenViewState>() {

    private var planList: List<PlanPresetModel>? = null

    private var estate: DealModel? = null

    private var showType: InfoShowType = InfoShowType.COMMON_INFO

    override fun defaultErrorViewStateHandler() =
        { e: ExceptionString -> DealScreenViewState.SomeError(e) }


    override fun bindIntents() {

        val fetchDeal = interactor.getDeal(dealId)
            .flatMap { deal ->
                interactor.getStatusList().map { DealScreenViewState.LoadDeal(deal,it) }
            }


        val state = restoreStateObservable
            .mergeWith(fetchDeal)
            .doOnError { Timber.e(it) }
            .subscribeOn(Schedulers.io())
            .observeOn(AndroidSchedulers.mainThread())

        subscribeViewState(state.cast(DealScreenViewState::class.java), DealScreen::render)
    }

    private fun langString(@StringRes id:Int) = context.resources.getString(id)

    private fun langString(@StringRes id:Int,vararg args:Any) =
            context.resources.getString(id,*args)

}

