package com.biganto.visual.roompark.presentation.screen.deals.util

import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.LinearLayout
import android.widget.TextView
import butterknife.BindView
import butterknife.ButterKnife
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.domain.model.StatusModel
import com.biganto.visual.roompark.presentation.screen.deals.DealPreviewModel
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonRecyclerAdapter
import com.biganto.visual.roompark.presentation.screen.settings.util.CommonViewHolder
import com.biganto.visual.roompark.util.extensions.toRubly
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressAnimationState
import com.biganto.visual.roompark.util.view_utils.status_progress_view.StatusProgressCeil
import com.google.android.material.textview.MaterialTextView

/**
 * Created by Vladislav Bogdashkin on 16.10.2019.
 */

class DealsListAdapter : CommonRecyclerAdapter<DealViewHolder,DealPreviewModel>() {
    override val vhKlazz = DealViewHolder::class
    override fun getVhLayout() = R.layout.deal_card_viewholder
}


class DealViewHolder(itemView: View) : CommonViewHolder<DealPreviewModel>(itemView) {


    @BindView(R.id.deal_card_header) lateinit var estateTitle: TextView
    @BindView(R.id.common_info_block) lateinit var commonInfo:View
    @BindView(R.id.start_tour_button) lateinit var startTour:View

    @BindView(R.id.info_ceil_1) lateinit var info1:View
    @BindView(R.id.info_ceil_2) lateinit var info2:View
    @BindView(R.id.info_ceil_3) lateinit var info3:View
    @BindView(R.id.info_ceil_4) lateinit var info4:View

    @BindView(R.id.deal_sum_value_text_view) lateinit var dealSum:TextView
    @BindView(R.id.deal_payed_value_text_view) lateinit var dealPayed:TextView
    @BindView(R.id.deal_to_pay_value_text_view) lateinit var dealSumToPay:TextView

    @BindView(R.id.progress_holder) lateinit var statusContainer:LinearLayout

    @BindView(R.id.statusContainer) lateinit var statusLayout:LinearLayout


    private fun typeName(type:String) = when(type){
        "flat" -> "КВАРТИРА"
        "parking" -> "МАШИНОМЕСТО"
        "storage" -> "КЛАДОВКА"
        else -> "ОБЪЕКТ"
    }

    init {
        ButterKnife.bind(this, itemView)
    }

    override fun onViewBound(model: DealPreviewModel) {
        val fullObjName = "${typeName(model.type)}\n№${model.name}"
        estateTitle.text = fullObjName
        renderCommonInfo(model)

        if (model.dealTourIds.isNullOrEmpty()) startTour.visibility = View.GONE
        else startTour.visibility = View.VISIBLE

        dealSum.text = model.dealSum.toRubly()
        dealPayed.text = model.dealPayout.toRubly()
        dealSumToPay.text = model.dealToPay.toRubly()

        statusContainer.deployProgressView(model.statusList,model.statusNo,false)

        val currentStatusText = LayoutInflater.from(itemView.context)
            .inflate(R.layout.status_title_textview,statusContainer,false)
                as MaterialTextView
        currentStatusText.text = model.statusList[model.statusNo-1].title
        statusLayout.addView(currentStatusText,0)
        statusLayout.invalidate()
    }

    private fun renderCommonInfo(info:DealPreviewModel){

        if (info.building == null) info1.visibility = View.GONE
        else { info1.title().text = "Корпус"; info1.text().text = info.building.toString()}

        if (info.section == null) info1.visibility = View.GONE
        else { info2.title().text = "Секция"; info2.text().text = info.section.toString()}

        if (info.floor == null) info1.visibility = View.GONE
        else { info3.title().text = "Этаж"; info3.text().text = info.floor.toString()}

        if (info.area == null) info1.visibility = View.GONE
        else { info4.title().text = "Общая, м²"; info4.text().text = info.area.toString()}
    }

    private fun View.title() = this.findViewById<TextView>(R.id.info_ceil_header)
    private fun View.text() = this.findViewById<TextView>(R.id.info_ceil_content)

}

private fun LinearLayout.deployProgressView(
    list: List<StatusModel>,
    currentStatus: Int,
    isVertical: Boolean
){
    this.removeAllViews()
    this.weightSum = list.size.toFloat()
    list.sortedBy { it.orderId }.forEach{
        this.addView(createStatusView(this,isVertical,it.orderId,list.size,currentStatus))
    }
    this.invalidate()
}

fun createStatusView(container: ViewGroup
                     , isVertical: Boolean
                     ,position: Int
                     ,statusCount: Int
                     ,lastStatusPassed:Int): StatusProgressCeil {
    val scale = container.resources.displayMetrics.density.toDouble()
    val param24dp = kotlin.math.ceil(24 * scale).toInt()
    val wrapContent = ViewGroup.LayoutParams.WRAP_CONTENT

    val ceil = LayoutInflater.from(container.context)
        .inflate(R.layout.status_view,container,false) as StatusProgressCeil
    ceil.layoutParams.height = if (isVertical) wrapContent else param24dp
    ceil.layoutParams.width = if (isVertical) param24dp else wrapContent
    (ceil.layoutParams as LinearLayout.LayoutParams).weight = 1f
    (ceil.layoutParams as LinearLayout.LayoutParams).gravity = Gravity.BOTTOM
    ceil.setHasEnd(position != statusCount)
    ceil.setHasStart(position > 1)
    ceil.setIsEnabled(position <= lastStatusPassed)
    ceil.setNext(position < lastStatusPassed)
    if (position > lastStatusPassed)
        ceil.setAnimState(StatusProgressAnimationState.DISABLE)
    ceil.invalidate()
    return ceil
}