package com.biganto.visual.roompark.presentation.screen.home.home_routing

import android.os.Bundle
import android.view.View
import androidx.annotation.IdRes
import androidx.annotation.NonNull
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.IBottomNavigation
import com.biganto.visual.roompark.base.ICollapsingToolBar
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.presentation.screen.deals.DealsScreenController
import com.biganto.visual.roompark.presentation.screen.favorites.FavoritesScreenController
import com.biganto.visual.roompark.presentation.screen.feeds.FeedsScreenController
import com.biganto.visual.roompark.presentation.screen.home.DaggerHomeScreenComponent
import com.biganto.visual.roompark.presentation.screen.home.HomeScreenPresenter
import com.biganto.visual.roompark.presentation.screen.home.HomeScreenViewState
import com.biganto.visual.roompark.presentation.screen.settings.SettingsScreenController
import com.biganto.visual.roompark.presentation.screen.to_flat.FindFlatScreenController
import com.bluelinelabs.conductor.Controller
import com.google.android.material.bottomnavigation.BottomNavigationView
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 14.02.2019.
 */
const val OPEN_CHILD_CONTROLLER_KEY="OPEN_HOME_SCREEN_CHILD_CONTROLLER_ID"

fun toPageBundle(@IdRes menuItemId:Int): Bundle {
    val b = Bundle()
    b.putInt(OPEN_CHILD_CONTROLLER_KEY, menuItemId)
    return b
}

class HomeBottomNavigationController(@IdRes toPage: Int = R.id.tab_feeds)
    : BottomNavigationController(toPageBundle(toPage)) {

    override fun injectDependencies() {
        getComponent()
    }

    override fun getStartupMenuId(): Int =
        args.getInt(OPEN_CHILD_CONTROLLER_KEY, R.id.tab_feeds)


    @Inject
    lateinit var tb: ICollapsingToolBar

    @Inject
    lateinit var bottomNavigation: IBottomNavigation

    @Inject
    override lateinit var injectedPresenter: HomeScreenPresenter


    fun getComponent() = DaggerHomeScreenComponent
        .factory()
        .create(RoomParkApplication.component, activity as RoomParkMainActivity)
        .inject(this)


    override fun render(viewState: HomeScreenViewState) {
        when (viewState) {
            is HomeScreenViewState.Idle -> render(viewState)
            is HomeScreenViewState.ToScreen -> render(viewState)
            is HomeScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: HomeScreenViewState.Idle) {

    }

    private fun render(viewState: HomeScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: HomeScreenViewState.ToScreen) {


//        snacky.showSnackBar("lul")
    }


    override fun getLayoutId() = R.layout.tab_routing_screen_view

    override val tag: String
        get() = "BottomNavigationImpl"


    override fun onViewBound(@NonNull view: View) {
        bottomNavigation.show()
        Timber.d(" onViewBound: ${this::class}")

        bottomNavigationView =
            ((activity as RoomParkMainActivity) as IBottomNavigation).bottomNavigation
        bottomNavigationView.visibility = BottomNavigationView.VISIBLE
        controllerContainer = view.findViewById(R.id.tabContainer)
        bottomNavigationView.setOnNavigationItemSelectedListener { item ->
            navigateTo(getControllerFor(item.itemId), false)
            true
        }

        router.backstack.clear()
        val toScreen = args.getInt(OPEN_CHILD_CONTROLLER_KEY, R.id.tab_feeds)
    }

    override fun getControllerFor(menuItemId: Int): Controller = when (menuItemId) {
        R.id.tab_feeds -> FeedsScreenController()
        R.id.tab_favorites -> FavoritesScreenController()
        R.id.tab_deals -> DealsScreenController()
        R.id.tab_look_flat -> FindFlatScreenController()
        R.id.tab_settings -> SettingsScreenController()
        else -> FeedsScreenController()
    }

    override fun isSupportedController(controller: Controller): Boolean {
        return true
    }

    override fun getIdForController(controller: Controller): Int {
        return when (controller) {
            is FeedsScreenController -> R.id.tab_feeds
            is FavoritesScreenController -> R.id.tab_favorites
            is DealsScreenController -> R.id.tab_deals
            is FindFlatScreenController -> R.id.tab_look_flat
            is SettingsScreenController -> R.id.tab_settings
            else -> 0
        }
    }

    override fun handleBack(): Boolean {
        /*
     * The childRouter should handleBack,
     * as this BottomNavigationController doesn't have a back step sensible to the user.
     */

        Timber.d("in back stack ")
        val childRouter = getChildRouter(currentlySelectedItemId)
        Timber.d("in back stack - ${childRouter?.backstackSize}")
        if (childRouter != null) {
            if (childRouter.backstackSize<2) return false

            if (childRouter.handleBack())
                navigateTo(getControllerFor(R.id.tab_feeds), false)

            return true

        } else {
            Timber.e(
                IllegalStateException(
                    "handleBack called with getChildRouter(currentlySelectedItemId) == null."
                ),
                "handleBack called with getChildRouter(currentlySelectedItemId) == null."
            )
        }
        return false
    }

}
