package com.biganto.visual.roompark.presentation.screen.photo

import android.os.Bundle
import android.view.View
import androidx.core.os.bundleOf
import androidx.viewpager2.widget.ViewPager2
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.IBottomNavigation
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.photo.util.PhotoViewerAdapter
import com.google.android.material.textview.MaterialTextView
import com.squareup.picasso.Picasso
import timber.log.Timber
import javax.inject.Inject


/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

const val SELECTED_PHOTO_KEY = "SELECTED_PHOTO_ID"

class PhotoScreenController :
    BigantoBaseController<PhotoScreenViewState
            , PhotoScreen
            , PhotoScreenPresenter>
   , PhotoScreen {

    constructor(args: Bundle):super(args)

    constructor(id: Int) : super(bundleOf(SELECTED_PHOTO_KEY to id))


    @BindView(R.id.photoTitle)
    lateinit var albumTitle: MaterialTextView

    @BindView(R.id.photo_frame)
    lateinit var photoViewPager: ViewPager2


    private fun bindRecycler() {
        photoViewPager.isNestedScrollingEnabled = false
        photoViewPager.offscreenPageLimit = 2
        photoViewPager.adapter = PhotoViewerAdapter()
    }

    override fun onViewBound(v: View) {
        toolBar.hideAll()
        bottomNavigation.hide()
        bindRecycler()
    }

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: PhotoScreenPresenter


//    @Inject
//    lateinit var snacky:ISnackBarProvider

    lateinit var rpActivity: RoomParkMainActivity

    fun getComponent() = DaggerPhotoScreenComponent.factory()
        .create(RoomParkApplication.component
            ,activity as RoomParkMainActivity
            ,args.getInt(SELECTED_PHOTO_KEY))
        .inject(this)

//    @Inject
//    lateinit var ac: RoomParkMainActivity

    @Inject
    lateinit var bottomNavigation: IBottomNavigation

    override fun render(viewState: PhotoScreenViewState) {
        Timber.d("render: $viewState")
        when(viewState){
            is PhotoScreenViewState.Idle -> render(viewState)
            is PhotoScreenViewState.PhotoSelected -> render(viewState)
            is PhotoScreenViewState.PhotoFetched -> render(viewState)
            is PhotoScreenViewState.PhotoListLoaded -> render(viewState)
            is PhotoScreenViewState.AldumFetched-> render(viewState)
            is PhotoScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: PhotoScreenViewState.Idle){

    }

    private fun render(viewState: PhotoScreenViewState.SomeError) =
        showError(viewState.exception)

    @Inject
    lateinit var picassoAsync:Picasso

    private fun render(viewState: PhotoScreenViewState.PhotoListLoaded) {
        (photoViewPager.adapter as PhotoViewerAdapter).setItems(
            viewState.list.asSequence().sortedBy{it.sort}.toList()
        )
        photoViewPager.currentItem = (photoViewPager.adapter as PhotoViewerAdapter)
            .getItemPosition(viewState.selectedId)
    }

    private fun render(viewState: PhotoScreenViewState.AldumFetched) {
        albumTitle.text = viewState.model.title
    }


    private fun render(viewState: PhotoScreenViewState.PhotoFetched){
        (photoViewPager.adapter as PhotoViewerAdapter).setItems(arrayListOf(viewState.model))
    }
    private fun render(viewState: PhotoScreenViewState.PhotoSelected){
//        (photoViewPager.adapter as PhotoViewerAdapter).setItems(arrayListOf(viewState.model))
    }



    override fun getLayoutId(): Int = R.layout.photo_view_screen

}