package com.biganto.visual.roompark.presentation.screen.splash

import android.view.View
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.IBottomNavigation
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.presentation.screen.auth.AuthScreenController
import com.biganto.visual.roompark.presentation.screen.home.home_routing.HomeBottomNavigationController
import com.bluelinelabs.conductor.RouterTransaction
import com.bluelinelabs.conductor.changehandler.FadeChangeHandler
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class SplashScreenController  :
    BigantoBaseController<SplashScreenViewState
        , SplashScreen
        , SplashScreenPresenter>()
   , SplashScreen {

    
    override fun onViewBound(v: View) {
        toolBar.hideAll()
        bottomNavigation.hide()
    }

    override fun injectDependencies() {
        getComponent()
    }

    @Inject
    override lateinit var injectedPresenter: SplashScreenPresenter


//    @Inject
//    lateinit var snacky:ISnackBarProvider

    lateinit var rpActivity: RoomParkMainActivity

    fun getComponent() = DaggerSplashScreenComponent
        .factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

//    @Inject
//    lateinit var ac: RoomParkMainActivity

    @Inject
    lateinit var bottomNavigation: IBottomNavigation

    override fun render(viewState: SplashScreenViewState) {
        when(viewState){
            is SplashScreenViewState.Idle -> render(viewState)
            is SplashScreenViewState.ToAuthScreen -> render(viewState)
            is SplashScreenViewState.ToHomeScreen -> render(viewState)
            is SplashScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: SplashScreenViewState.Idle){

    }

    private fun render(viewState: SplashScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: SplashScreenViewState.ToAuthScreen){
        router.pushController(RouterTransaction.with(AuthScreenController())
            .popChangeHandler(FadeChangeHandler())
            .pushChangeHandler(FadeChangeHandler())
        )
    }

    private fun render(viewState: SplashScreenViewState.ToHomeScreen){
        router.setRoot(RouterTransaction.with(HomeBottomNavigationController())
            .popChangeHandler(FadeChangeHandler(200L))
            .pushChangeHandler(FadeChangeHandler(300L))
        )
    }


    override fun getLayoutId(): Int = R.layout.splash_screen


}