package com.biganto.visual.roompark.presentation.screen.to_flat

import android.view.KeyEvent
import android.view.View
import android.widget.EditText
import android.widget.TextView
import butterknife.BindView
import com.biganto.visual.roompark.R
import com.biganto.visual.roompark.base.RoomParkApplication
import com.biganto.visual.roompark.base.RoomParkMainActivity
import com.biganto.visual.roompark.conductor.BigantoBaseController
import com.biganto.visual.roompark.domain.model.TourRequestModel
import com.biganto.visual.roompark.presentation.screen.feeds.EstateTabModel
import com.google.android.material.button.MaterialButton
import com.google.android.material.tabs.TabLayout
import com.google.android.material.textfield.TextInputLayout
import com.jakewharton.rxbinding3.view.keys
import io.reactivex.Observable
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 30.09.2019.
 */

class FindFlatScreenController  :
    BigantoBaseController<FindFlatScreenViewState
        , FindFlatScreen
        , FindFlatScreenPresenter>()
    , FindFlatScreen {


    override fun getFlat(): Observable<TourRequestModel> =
        flatNumberEditor.keys { it.keyCode == KeyEvent.KEYCODE_ENTER }
            .doOnNext { flatNumberEditor.hideKeyboard() }
            .map {
                TourRequestModel(
                    estateTabs[flatTabs.selectedTabPosition].building
                    , flatNumberInput.editText?.text.toString()
                )
            }

    override fun startTour(): Observable<String> = Observable.just("1")

    override fun injectDependencies() {
        getComponent()
    }

    private fun setToolbar(){
        toolBar.showAll()
        toolBar.appBar.setExpanded(false,true)
        toolBar.collapsingToolbarLayout.title = "СМОТРЕТЬ\nКВАРТИРУ"
        toolBar.appBar.liftOnScrollTargetViewId = R.id.favorites_cards_recycler_view
        toolBar.appBar.setLiftable(true)
        toolBar.appBar.setLifted(true)
        toolBar.appBarScrollable(false)
    }


    @BindView(R.id.flat_estate_tabs)
    lateinit var flatTabs: TabLayout

    @BindView(R.id.flatNumberInputer)
    lateinit var flatNumberInput: TextInputLayout

    @BindView(R.id.flatNumberEditText)
    lateinit var flatNumberEditor: EditText

    @BindView(R.id.findFlatButton)
    lateinit var fubdFlatButton: MaterialButton

    @Inject
    override lateinit var injectedPresenter: FindFlatScreenPresenter

    private val estateTabs = arrayListOf(
        EstateTabModel("Д1",1),
        EstateTabModel("Д2",2),
        EstateTabModel("Д3",3)
    )

    override fun onViewBound(v: View) {
        setToolbar()
        estateTabs.forEach {estate ->
            val tab = flatTabs.newTab()
                .setCustomView(R.layout.to_flat_tab_view)
            tab.customView
                ?.let {
                    it.findViewById<TextView>(R.id.tab_title)?.text = estate.title

                    it.findViewById<TextView>(R.id.tab_divider)?.visibility =
                        if (estateTabs.indexOf(estate) == estateTabs.size - 1) View.GONE
                        else View.VISIBLE
                }

            flatTabs.addTab(tab)
        }
    }

    override fun render(viewState: FindFlatScreenViewState) {
        super.render(viewState)
        Timber.d("Render state $viewState")
        when(viewState){
            is FindFlatScreenViewState.Idle -> render(viewState)
            is FindFlatScreenViewState.StartTour -> render(viewState)
            is FindFlatScreenViewState.SomeError -> render(viewState)
        }
    }

    private fun render(viewState: FindFlatScreenViewState.SomeError) =
        showError(viewState.exception)

    private fun render(viewState: FindFlatScreenViewState.Idle){

    }

    private fun getComponent() = DaggerFindFlatScreenComponent.factory()
        .create(RoomParkApplication.component,activity as RoomParkMainActivity)
        .inject(this)

    override fun getLayoutId(): Int = R.layout.find_flat_screen


}